unit SB_ImageCommon;

interface

uses
	Windows, Messages, SysUtils, Classes, Controls, Graphics, Math;

ResourceString
	SVersion = 'Version: 0.1.20';
  SCopyright = 'Copyright (c) 2003 JSC & I Corp. All Rights Reserved.';

Type

	{ Other dynamic arrays }
  TArrayOfByte = Array of Byte;
  TArrayOfInteger = Array of Integer;
  TArrayOfArrayOfInteger = Array of TArrayOfInteger;
  TArrayOfSingle = Array of Single;

  TRGBArray = ARRAY[0..32767] OF TRGBTriple;
  pRGBArray = ^TRGBArray;

  	{ Calculates Hue, Luminance and Saturation for the clrRGB value }
	procedure ColorRGBToHLS(clrRGB: COLORREF; var Hue, Luminance, Saturation: Word);
  	{ Calculates a color given Hue, Luminance and Saturation values }
  	Function ColorHLSToRGB(Hue, Luminance, Saturation: Word): TColorRef;
  	Function  IntToByte(i: Integer): Byte;
  	Function  TrimInt(i, Min, Max: Integer): Integer;
  	Function HueToRGB(Lum, Sat, Hue: Double): Integer;
	Function Adjust(Ray : Array of Integer; Z : Word; Src : TBitmap): TBitmap;
	Function Set255(Clr : Integer) : Integer;

const
  HLSMAX = 240;            // H,L, and S vary over 0-HLSMAX
  RGBMAX = 255;            // R,G, and B vary over 0-RGBMAX
                           // HLSMAX BEST IF DIVISIBLE BY 6
                           // RGBMAX, HLSMAX must each fit in a byte.

  { Hue is undefined if Saturation is 0 (grey-scale)
    This value determines where the Hue scrollbar is
    initially set for achromatic colors }
  HLSUndefined = (HLSMAX*2/3);

implementation

threadvar
  CachedRGBToHLSclrRGB: COLORREF;
  CachedRGBToHLSHue: WORD;
  CachedRGBToHLSLum: WORD;
  CachedRGBToHLSSat: WORD;



//****************************************************************************
// Calculates Hue, Luminance and Saturation for the clrRGB value
//****************************************************************************
Procedure ColorRGBToHLS(clrRGB: COLORREF; var Hue, Luminance, Saturation: Word);
Var
  H, L, S: Double;
  R, G, B: Word;
  cMax, cMin: Double;
  Rdelta, Gdelta, Bdelta: Extended; // intermediate value: % of spread from max
Begin
  If clrRGB = CachedRGBToHLSclrRGB then
  Begin
  	Hue := CachedRGBToHLSHue;
    Luminance := CachedRGBToHLSLum;
    Saturation := CachedRGBToHLSSat;
    Exit;
  End;
  R := GetRValue(clrRGB);
  G := GetGValue(clrRGB);
  B := GetBValue(clrRGB);

  // Calculate Lightness 
  cMax := Math.Max(Math.Max(R, G), B);
  cMin := Math.Min(Math.Min(R, G), B);
  L := ( ((cMax + cMin) * HLSMAX) + RGBMAX ) / ( 2 * RGBMAX);
  If cMax = cMin then  // r=g=b --> achromatic case 
  Begin                // saturation
    Hue := Round(HLSUndefined);
//    pwHue := 160;      { MS's ColorRGBToHLS always defaults to 160 in this case }
    Luminance := Round(L);
    Saturation := 0;
  End
  Else                 // chromatic case
  Begin
    // Saturation
    If L <= HLSMAX/2 then
    	S := ( ((cMax-cMin)*HLSMAX) + ((cMax+cMin)/2) ) / (cMax+cMin)
    Else
    	S := ( ((cMax-cMin)*HLSMAX) + ((2*RGBMAX-cMax-cMin)/2) ) / (2*RGBMAX-cMax-cMin);

    // hue
    Rdelta := ( ((cMax-R)*(HLSMAX/6)) + ((cMax-cMin)/2) ) / (cMax-cMin);
    Gdelta := ( ((cMax-G)*(HLSMAX/6)) + ((cMax-cMin)/2) ) / (cMax-cMin);
    Bdelta := ( ((cMax-B)*(HLSMAX/6)) + ((cMax-cMin)/2) ) / (cMax-cMin);

    If (R = cMax) then
      	H := Bdelta - Gdelta
    Else if (G = cMax) then
      	H := (HLSMAX/3) + Rdelta - Bdelta
    Else // B == cMax
      	H := ((2 * HLSMAX) / 3) + Gdelta - Rdelta;

    if (H < 0) then
      	H := H + HLSMAX;
    if (H > HLSMAX) then
      	H := H - HLSMAX;
    Hue := Round(H);
    Luminance := Round(L);
    Saturation := Round(S);
  End;
  CachedRGBToHLSclrRGB := clrRGB;
  CachedRGBToHLSHue := Hue;
  CachedRGBToHLSLum := Luminance;
  CachedRGBToHLSSat := Saturation;
End;
//****************************************************************************
// Calculates a color given Hue, Luminance and Saturation values
//****************************************************************************
Function ColorHLSToRGB(Hue, Luminance, Saturation: Word): TColorRef;

  function RoundColor(Value: Double): Integer;
  Begin
    if Value > 255 then
      Result := 255
    else
      Result := Round(Value);
  End;

Var
  R,G,B: Double;               // RGB component values
  Magic1,Magic2: Double;       // calculated magic numbers (really!) 
Begin
  If (Saturation = 0) then
  Begin            // achromatic case 
     R := (Luminance * RGBMAX)/HLSMAX;
     G := R;
     B := R;
     if (Hue <> HLSUndefined) then
       ;// ERROR 
  End
  Else
  Begin            // chromatic case
     // set up magic numbers
     If (Luminance <= (HLSMAX/2)) then
        Magic2 := (Luminance * (HLSMAX + Saturation) + (HLSMAX/2)) / HLSMAX
     Else
        Magic2 := Luminance + Saturation - ((Luminance * Saturation) + (HLSMAX/2)) / HLSMAX;
     Magic1 := 2 * Luminance - Magic2;

     // get RGB, change units from HLSMAX to RGBMAX
     R := (HueToRGB(Magic1,Magic2,Hue+(HLSMAX/3))*RGBMAX + (HLSMAX/2))/HLSMAX;
     G := (HueToRGB(Magic1,Magic2,Hue)*RGBMAX + (HLSMAX/2)) / HLSMAX;
     B := (HueToRGB(Magic1,Magic2,Hue-(HLSMAX/3))*RGBMAX + (HLSMAX/2))/HLSMAX;
  end;
  Result := RGB(RoundColor(R), RoundColor(G), RoundColor(B));
end;
//******************************************************************************
// Convert Integer to Byte data type
//******************************************************************************
Function  IntToByte(i: Integer): Byte;
Begin
  If      i > 255 Then  Result := 255
  Else If i < 0   Then  Result := 0
  Else                  Result := i;
End;

//******************************************************************************
// Truncate overflow or underflow value of specified range.
//******************************************************************************
Function  TrimInt(i, Min, Max: Integer): Integer;
begin
  If      i>Max then Result := Max
  Else If i<Min then Result := Min
  Else               Result := i;
end;
//****************************************************************************
// Hue to RGB color.
//****************************************************************************
Function HueToRGB(Lum, Sat, Hue: Double): Integer;
var
  ResultEx: Double;
begin
  // range check: note values passed add/subtract thirds of range
  if (hue < 0) then
     hue := hue + HLSMAX;

  if (hue > HLSMAX) then
     hue := hue - HLSMAX;

  // return r,g, or b value from this tridrant
  if (hue < (HLSMAX/6)) then
    ResultEx := Lum + (((Sat-Lum)*hue+(HLSMAX/12))/(HLSMAX/6))
  else if (hue < (HLSMAX/2)) then
    ResultEx := Sat
  else if (hue < ((HLSMAX*2)/3)) then
    ResultEx := Lum + (((Sat-Lum)*(((HLSMAX*2)/3)-hue)+(HLSMAX/12))/(HLSMAX/6))
  else
    ResultEx := Lum;
  Result := Round(ResultEx);
End;

//****************************************************************************
// Adjust - Convolution processing.
//****************************************************************************
{This is my version of a 3x3 convolution, for 24bit bitmaps....}
Function Adjust(Ray : Array of Integer; Z : Word; Src : TBitmap)
		: TBitmap;
Var
  tmpBmp: TBitmap;
  DstY, A, B, C : pRGBArray;
  x, y : Integer;
begin
	Result := TBitmap.Create;
  Result.PixelFormat := pf24bit;
  Result.Width := Src.Width;
  Result.Height := Src.Height;

  tmpBmp := TBitmap.Create;
  tmpBmp.Width :=  Src.Width+2;  //Add a box around the outside...
  tmpBmp.Height := Src.Height+2;

  tmpBmp.PixelFormat := pf24bit;
  Src.PixelFormat    := pf24bit;
  DstY := tmpBmp.ScanLine[0];   //Copy top corner pixels
  A := Src.ScanLine[0];
  DstY[0] := A[0];
  DstY[tmpBmp.Width - 1] := A[Src.Width - 1];
  //Copy bottom line to our top - trying to remain seamless...
  tmpBmp.Canvas.CopyRect(RECT(1,0,tmpBmp.Width - 1,1),Src.Canvas,
          RECT(0,Src.Height - 1,Src.Width,Src.Height-2));

  DstY := tmpBmp.ScanLine[tmpBmp.Height - 1]; //Copy bottom corner pixels
  A := Src.ScanLine[Src.Height - 1];
  DstY[0] := A[0];
  DstY[tmpBmp.Width - 1] := A[Src.Width - 1];
  //Copy top line to our bottom
  tmpBmp.Canvas.CopyRect(RECT(1,tmpBmp.Height-1,tmpBmp.Width - 1,tmpBmp.Height),
         Src.Canvas,RECT(0,0,Src.Width,1));
  //Copy left to our right
  tmpBmp.Canvas.CopyRect(RECT(tmpBmp.Width-1,1,tmpBmp.Width,tmpBmp.Height-1),
         Src.Canvas,RECT(0,0,1,Src.Height));
  //Copy right to our left
  tmpBmp.Canvas.CopyRect(RECT(0,1,1,tmpBmp.Height-1),
         Src.Canvas,RECT(Src.Width - 1,0,Src.Width,Src.Height));
  //Now copy main rectangle
  tmpBmp.Canvas.CopyRect(RECT(1,1,tmpBmp.Width - 1,tmpBmp.Height - 1),
    Src.Canvas,RECT(0,0,Src.Width,Src.Height));
  //bmp now enlarged and copied
  For X := 0 to Src.Height - 1 do
  Begin
		DstY := Result.ScanLine[x];
    A := tmpBmp.ScanLine[x];     //old x-1
    B := tmpBmp.ScanLine[x+1];   //old x
    C := tmpBmp.ScanLine[x+2];   //old x+1
  //Now do the main piece
    For Y := 1 to (tmpBmp.Width - 2) do
    Begin
      	DstY[y-1].rgbtRed := Set255(
          ((A[y-1].rgbtRed*Ray[0])+ (A[y].rgbtRed*Ray[1]) + (A[y+1].rgbtRed*ray[2]) +
          (B[y-1].rgbtRed*Ray[3]) + (B[y].rgbtRed*Ray[4]) + (B[y+1].rgbtRed*ray[5])+
          (C[y-1].rgbtRed*Ray[6]) + (C[y].rgbtRed*Ray[7]) + (C[y+1].rgbtRed*ray[8])) Div z
          );
      	DstY[y-1].rgbtBlue := Set255(
          ((A[y-1].rgbtBlue*Ray[0])+ (A[y].rgbtBlue*Ray[1]) + (A[y+1].rgbtBlue*ray[2]) +
          (B[y-1].rgbtBlue*Ray[3]) + (B[y].rgbtBlue*Ray[4]) + (B[y+1].rgbtBlue*ray[5])+
          (C[y-1].rgbtBlue*Ray[6]) + (C[y].rgbtBlue*Ray[7]) + (C[y+1].rgbtBlue*ray[8])) Div z
          );
      	DstY[y-1].rgbtGreen := Set255(
          ((A[y-1].rgbtGreen*Ray[0])+(A[y].rgbtGreen*Ray[1]) + (A[y+1].rgbtGreen*ray[2]) +
          (B[y-1].rgbtGreen*Ray[3]) +(B[y].rgbtGreen*Ray[4]) + (B[y+1].rgbtGreen*ray[5])+
          (C[y-1].rgbtGreen*Ray[6]) +(C[y].rgbtGreen*Ray[7]) + (C[y+1].rgbtGreen*ray[8])) Div z
          );
		End; // of for y.
  End; // of for x.
end;
{This just forces a value to be 0 - 255 for rgb purposes.  I used asm in an
 attempt at speed, but I don't think it helps much.}
Function Set255(Clr : Integer) : Integer;
asm
  MOV  EAX,Clr
  CMP  EAX,254
  JG   @SETHI
  CMP  EAX,1
  JL   @SETLO
  RET
@SETHI:
  MOV  EAX,255
  RET
@SETLO:
  MOV  EAX,0
End;

end.
