//****************************************************************************
//
// Borland Delphi Visual Component Library
//
// Image Effect Processing View Component
//
//  : 2003 10
// : ȯ, C2.
//
// This is part of SB_ImagePro Project File.
// Copyright 2003 by JSC&I (Mr.Bae). All rights reserved.  
//****************************************************************************
unit SB_ImageEffectView;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms, Controls, Graphics,
  ExtCtrls, Math, SB_ImageCommon, SB_ImageEffect;


Type


	TSBImage32 = Class(TImage)
  Private
  	MouseDragPos,
    MouseMovePos : TPoint;
  	MouseDragging: Boolean;

  Public

    Constructor Create(AOwner:TComponent); override;
    Destructor Destroy; override;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
                        X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
              X, Y: Integer); override;
  Published

  End;

  TSBImageEffectView = class(TGraphicControl)
  private
    FBitmap        : TBitmap; 	// ȭ鿡  ̹.
    PicRect        : TRect;     // ̹ ũ
    ShowRect       : TRect;		// Ʈ ȭ ũ.
    LeftTop        : TPoint;    // ȭ鿡  ̹ ġ(Left, Top)
    CropRect       : TRect;     // Cropping Rect ´.
  	FAutoSize	   : Boolean;   //  ̹ ũ  
    FCropMode      : Boolean;   // Cropping Mode  (콺 ̺Ʈó)
	FShowBorder    : Boolean;	//   
    FBorderWidth   : Integer;   //  β
    FForceRepaint  : Boolean;	//  Drawing.
    FMouse         : (mNone, mDrag, mZoom, mCrop);	// 콺 ¸
    FProportional  : boolean;	// ̹ 
    FDblClkEnable  : boolean;   //  
    StartX, StartY,
    OldX, OldY     : integer;
    FVersion: String;


    Function  GetProcessBitmap: TBitmap;
    procedure SetShowBorder(S:boolean);
    procedure SetBitmap(B:TBitmap);
    procedure SetBorderWidth(w:integer);
    procedure SetProportional(b:boolean);
    procedure SetAutoSize(const Value: Boolean);
    Procedure MagnifyCroppingArea(Const XYSize: TRect);
    Procedure ResizeFitSize();
    Procedure ResetImageEffect();
    procedure SetVersion(const Value: String);
    function GetUseOriginalBitmap: boolean;
    procedure SetUseOriginalBitmap(const Value: boolean);
    function GetBitmap: TBitmap;
    procedure SetCropMode(const Value: Boolean);

  protected
    procedure Paint; override;
    procedure DblClick; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
                        X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
              X, Y: Integer); override;
  public
    Effect: TSBImageEffect;

    constructor Create(AOwner:TComponent); Override;
    destructor Destroy; override;

    Function  GetCropRect(): TRect;

    Procedure Exe_Invert;
    Procedure Exe_AddColorNoise   (Amount:Integer);
    Procedure Exe_AddMonoNoise    (Amount:Integer);
    Procedure Exe_AntiAlias;
    Procedure Exe_Contrast        (Amount:Integer);
    Procedure Exe_GrayScale       ;
    Procedure Exe_Brightness       (Amount:Integer);
    Procedure Exe_Darkness        (Amount:Integer);
    Procedure Exe_Saturation      (Amount:Integer);
    Procedure Exe_SplitBlur       (Amount:Integer);
    Procedure Exe_GaussianBlur    (Amount:Integer);
    Procedure Exe_Tile            (Amount: integer);
    Procedure Exe_Emboss;
    Procedure Exe_Solarize        (Amount: integer);
    Procedure Exe_Posterize       (Amount: integer);
  	Procedure Exe_FlipHorizontal  ;
    Procedure Exe_FlipVertical    ;
    Procedure Exe_ZoomXY		  (ScaleX, ScaleY: Single; AntiAlias: Boolean);
    Procedure Exe_Crop            (Bounds: TRect);
    Procedure Exe_Blur            ;
    Procedure Exe_Sharpen         ;
    Procedure Exe_Rotate          (Angle: Integer; AntiAlias: Boolean);
    procedure Exe_FishEye         (Amount: Integer);
    procedure Exe_Mosaic          (Amount: Integer);
    procedure Exe_Twist           (Amount: Integer);
    procedure Exe_Splitlight      (Amount: Integer);
    procedure Exe_SpotLight       (Amount: integer; Spot: TRect);
    procedure Exe_Trace           (Amount: integer);
    procedure Exe_Rotate90        (Angle: integer);
    procedure Exe_Zoom            (Scale: Single; AntiAlias: Boolean);


  published
  	Property AutoSize: Boolean
    			Read FAutoSize Write SetAutoSize Default True;
    property ShowBorder : boolean
    			 Read FShowBorder
                 write SetShowBorder default true;
    property KeepAspect : boolean
                 Read FProportional
                 Write SetProportional default true;
    property Bitmap : TBitmap
                 Read GetBitmap
                 Write SetBitmap;
    property BorderWidth : integer
                 read FBorderWidth
                 write SetBorderWidth default 7;
    property CropMode: Boolean
    			Read FCropMode Write SetCropMode default False;
    property ForceRepaint : Boolean
                 read FForceRepaint
                 write FForceRepaint default true;
    property DblClkEnable : Boolean
                 read FDblClkEnable
                 write FDblClkEnable default False;
    property UseOriginalBitmap: boolean
    			Read GetUseOriginalBitmap Write SetUseOriginalBitmap;
    property Version: String
    			Read FVersion Write SetVersion;

    property Align;
    property Width;
    property Height;
    property Top;
    property Left;
    property Visible;
    property Hint;
    property ShowHint;

    Property OnClick;
    Property OnDragDrop;
    Property OnMouseDown;
    Property OnMouseMove;
    Property OnMouseUp;
    Property OnMouseWheel;

  end;


implementation

uses Types;

Constructor TSBImageEffectView.Create(AOwner:TComponent);
begin
  inherited;

  FAutoSize := True;
  FBorderWidth := 7;
  FCropMode := False;
  FDblClkEnable := False;
  FForceRepaint := True;
  FMouse := mNone;
  FProportional := True;
  FShowBorder := True;
  Width:=100; Height:=150;

  FBitmap := TBitmap.Create;
  FBitmap.Width := Width;
  FBitmap.Height := Height;

  LeftTop := Point(0, 0);
  PicRect := FBitmap.Canvas.ClipRect;

  ControlStyle := ControlStyle+[csOpaque];
  Canvas.Brush.Color := clBlack;

  Effect := TSBImageEffect.Create;
  Effect.UseSource := True; // ̹ ؼ ó 

  // Get version Info.
  FVersion := SVersion;
end;

destructor TSBImageEffectView.Destroy;
begin
  Effect.Free;
  FBitmap.Free;

	inherited;
end;
//****************************************************************************
// Ʈѿ ̹  ׸.
//****************************************************************************
procedure TSBImageEffectView.Paint;
var
	Bmp: TBitmap;
  Coef:Double;
begin
  Bmp := TBitmap.Create;
  Bmp.Width := Width;
  Bmp.Height := Height;

  CropRect := Rect(0,0,0,0);
  If Not FShowBorder then
  	ShowRect := ClientRect
  Else ShowRect := Rect(ClientRect.Left, ClientRect.Top,
  			ClientRect.Right-FBorderWidth, ClientRect.Bottom-FBorderWidth);
  With PicRect do
  Begin
  	If Right=0 then Right := FBitmap.Width;
    If Bottom=0 then Bottom := FBitmap.Height;
  End;

  //If FProportional Then
  Begin
   	Bmp.Canvas.CopyMode:=cmSrcCopy;
  	Bmp.Canvas.CopyRect(ShowRect, FBitmap.Canvas,
    		Rect(LeftTop.X, LeftTop.Y,
            LeftTop.X+ShowRect.Right, LeftTop.Y+ShowRect.Bottom));

  	If FShowBorder Then
    Begin
     	Bmp.Canvas.Brush.Color := clWhite;
     	Bmp.Canvas.Rectangle(Width-BorderWidth, 0, Width, Height-BorderWidth); // vertical
     	Bmp.Canvas.Rectangle(0, Height-BorderWidth, Width-BorderWidth, Height); // horizontal
			// Fill Right-Bottom Edge.
     	Bmp.Canvas.Brush.Color := clSilver;
     	Bmp.Canvas.Rectangle(Width-BorderWidth-1, Height-BorderWidth-1, Width, Height);

        //horizontal zoom indicator
        Coef := (Width-BorderWidth) / FBitmap.Width;
        Bmp.Canvas.Rectangle( Round(LeftTop.X*Coef)+2, Height-BorderWidth+2,
        		Round((LeftTop.X+ShowRect.Right)*Coef)+2, Height-2);
     	//vertical zoom indicator
     	Coef := (Height-BorderWidth) / FBitmap.Height;
     	Bmp.Canvas.Rectangle( Width-BorderWidth+2, Round(LeftTop.Y*Coef)+2,
        		Width-2, Round((LeftTop.Y+ShowRect.Bottom)*Coef)-2);
   	End;
   	Canvas.CopyMode := cmSrcCopy;
  	Canvas.Draw(0,0,Bmp);
  	Bmp.Free;
  End;
End;
//****************************************************************************
// Mouse Down Event.
//****************************************************************************
procedure TSBImageEffectView.MouseDown(Button: TMouseButton; Shift: TShiftState;
		X, Y: Integer);
begin
	If (CropRect.Right-CropRect.Left <> 0) OR
    		(CropRect.Bottom-CropRect.Top <> 0) Then
    Begin
    	Canvas.DrawFocusRect( CropRect );
        CropRect := Rect(0,0,0,0);
    End;
	// Ʈ  Ŭ ʾҴٸ, ExitѴ.
	If Not PtInRect(ShowRect,Point(X,Y)) and
      Not PtInRect(Rect(ShowRect.Right, ShowRect.Bottom,
                        Width,Height), Point(X,Y)) Then Exit;
	//   콺 ŬϿٸ
  If PtInRect(Rect(ShowRect.Right, ShowRect.Bottom,
                    Width, Height), Point(X,Y)) Then
  Begin
  	DblClick;
    Exit;
  End;
  //here click is in the picture area only
  StartX := X; OldX := X;
  StartY := Y; OldY := Y;
  // 콺 ưŬ
  If mbRight = Button Then
  Begin
  	MouseCapture := True;
  	If (FCropMode) Then
    	FMouse := mCrop
    Else
	    FMouse := mZoom;
    Canvas.Pen.Mode := pmNot;
  End Else // 콺 ¹ưŬ(if user click mouse left-button)
  Begin
    FMouse := mDrag;
    Screen.Cursor := crHandPoint;
  End;
	Inherited;
end;
//****************************************************************************
// Mouse Move Event - Zoom & Drag 带 óѴ.
//****************************************************************************
procedure TSBImageEffectView.MouseMove(Shift: TShiftState; X, Y: Integer);
Var
	NewX, NewY,
  W, H: Integer;
begin
  If FMouse = mNone Then Exit;
  If ([ssLeft, ssRight] <> Shift) Then
  	Screen.Cursor := crDefault;
  If ( (FMouse = mZoom) OR (FMouse = mCrop) ) And (Shift = [ssRight]) then // Zoom 
  Begin
  	Canvas.DrawFocusRect( Rect(Min(StartX,OldX), Min(StartY,OldY),
    		Max(StartX,OldX), Max(StartY,OldY)) );
    OldX := X; OldY := Y;
    Canvas.DrawFocusRect( Rect(Min(StartX,X), Min(StartY,Y),
    		Max(StartX,X), Max(StartY,Y)) );
  End;
  // Drag 
  If (FMouse = mDrag) And (Shift = [ssLeft]) then
  Begin
  	// Calculate LeftTop Point
    W := Width - BorderWidth;
    H := Height - BorderWidth;
    NewX := Round(((X-OldX) * PicRect.Right) / W);
    NewY := Round(((Y-OldY) * PicRect.Bottom) / H);

    If (LeftTop.X + NewX >= 0) And (LeftTop.X + NewX < PicRect.Right-W+1) Then
        	LeftTop.X := LeftTop.X + NewX;
    If (LeftTop.Y + NewY >= 0) And (LeftTop.Y + NewY < PicRect.Bottom-H+1) Then
    	LeftTop.Y := LeftTop.Y + NewY;

		OldX := X; OldY := Y;

    If FForceRepaint Then 	Repaint
    Else 					Invalidate;
    Canvas.Brush.Style := bsClear;
    //Canvas.TextOut(ShowRect.Left+2, ShowRect.Top+ShowRect.Bottom-16, IntToStr(LeftTop.X));
    //Canvas.TextOut(ShowRect.Left+32, ShowRect.Top+ShowRect.Bottom-16, IntToStr(LeftTop.Y));
    Canvas.Brush.Style := bsSolid;
  End;
	Inherited;
end;
//****************************************************************************
// MouseUp ̺Ʈ - Zoom带 óѴ. Mouse Mode ʱȭѴ.
//****************************************************************************
procedure TSBImageEffectView.MouseUp(Button: TMouseButton; Shift: TShiftState;
		X, Y: Integer);
Var
	Bmp: TBitmap;
begin

  If FMouse = mNone Then Exit;
  If (Abs(X- StartX) < 5) OR (Abs(Y - StartY) < 5) Then Exit;
  If X > ShowRect.Right Then X := ShowRect.Right;
  If Y > ShowRect.Bottom Then Y := ShowRect.Bottom;
  // Zoom  ó
  If FMouse = mZoom Then
  Begin
  	// Calculate Scale and Resize image.
    MagnifyCroppingArea(Rect(StartX, StartY, X, Y));
    FMouse := mNone;
    MouseCapture := False;
    Invalidate;
  End;
  // Crop Mode processing.
  If FMouse = mCrop Then
  Begin
  	// Calculate BoundsRect.
    CropRect := Rect(Min(StartX,X), Min(StartY,Y), Max(StartX,X), Max(StartY,Y));
    FMouse := mNone;
  End;
  // Drag ó ʱȭ
  If FMouse = mDrag then
  Begin
    FMouse := mNone;
    Canvas.Pen.Mode := pmCopy;
    Screen.Cursor := crDefault;
    MouseCapture;
    Invalidate;
  End;
	Inherited;
end;
//****************************************************************************
//  ǵ
//****************************************************************************
procedure TSBImageEffectView.DblClick;
begin
  Effect.UsedBmp.Assign(Effect.SourceBmp); // ־ .
  FBitmap.Assign(Effect.SourceBmp);
  PicRect := Rect(0,0,FBitmap.Width, FBitmap.Height);
  LeftTop := Point(0,0);
  If Not FAutoSize Then
  	ResizeFitSize;
  Invalidate;
end;
//****************************************************************************
// ̹ εϱ
//****************************************************************************
procedure TSBImageEffectView.SetBitmap(B: TBitmap);
begin
  Effect.SourceBmp.Assign(B);
  DblClick();
end;
//****************************************************************************
//  
//****************************************************************************
procedure TSBImageEffectView.SetBorderWidth(w:integer);
begin
   FBorderWidth := w;
   Invalidate;
end;
//****************************************************************************
//  ̱ 
//****************************************************************************
procedure TSBImageEffectView.SetShowBorder(s:boolean);
begin
   FShowBorder := s;
   Invalidate;
end;
//****************************************************************************
//  ̹    
//****************************************************************************
procedure TSBImageEffectView.SetProportional(b:boolean);
begin
   FProportional := B;
   Invalidate;
end;
//****************************************************************************
// ó  ̹ ϱ
//****************************************************************************
Function TSBImageEffectView.GetProcessBitmap: TBitmap;
begin
  Result := TBitmap.Create;
  Result.Assign(FBitmap);
end;
//****************************************************************************
// Set AutoSize ɼ 
//****************************************************************************
procedure TSBImageEffectView.SetAutoSize(Const Value: Boolean);
begin
  FAutoSize := Value;
  If Not Value Then
	Begin
		SetProportional(Value); // AutoSize  󰡷μ   
  	ResizeFitSize;
  End;
end;
//****************************************************************************
// Stretch Image to control size.
//****************************************************************************
procedure TSBImageEffectView.ResizeFitSize;
begin
	MagnifyCroppingArea(PicRect);
end;
//****************************************************************************
// Magnify Image with cropping area of it.
//****************************************************************************
procedure TSBImageEffectView.MagnifyCroppingArea(const XYSize: TRect);
Var
  W, H: Integer;
	ScaleX, ScaleY, PicX, PicY: Double;
  Bmp: TBitmap;
begin
  //Calculate new PicRect
  W := Width - BorderWidth;
  H := Height - BorderWidth;
  // Cropping  Ȯ  
  PicX := W * PicRect.Right / Abs(XYSize.Right-XYSize.Left);
  PicY := H * PicRect.Bottom / Abs(XYSize.Bottom-XYSize.Top);
  // AutoSize False FProportional  True ɼ .
  If FProportional Then
	Begin
		ScaleX := Min(PicX / PicRect.Right, PicY / PicRect.Bottom);
  	ScaleY := ScaleX;
  End Else
  Begin
	  ScaleX := PicX / PicRect.Right;
	  ScaleY := PicY / PicRect.Bottom;

  End;

  If (GetUseOriginalBitmap) Then
  Begin
      // ȭ ̹ .
      Bmp := TBitmap.Create;
      Bmp.Assign(Effect.SourceBmp);
      // Bitmap scaleŭ ȮѴ.
      Effect.SourceBmp.Assign(Bmp);
  End;
  FBitmap.Assign( Effect.Effect_ZoomXY(ScaleX * 100, ScaleY * 100,false) );
  If (GetUseOriginalBitmap) Then
  Begin
    Effect.SourceBmp.Assign(Bmp);
  	Bmp.Free;
  End;

  // Ȯ ̹ ũ⸦ Ѵ.
  PicRect := Rect(0, 0, FBitmap.Width, FBitmap.Height);
  // Ȯ ̹ ShowRect  ġ(LeftTop) Ѵ.
  LeftTop := Point( Round(XYSize.Left * ScaleX), Round(XYSize.Top * ScaleY) );

end;
//****************************************************************************
// ̹ ȿ ϰ  ̹ Ѵ.
//****************************************************************************
procedure TSBImageEffectView.ResetImageEffect();
begin
  FBitmap.Assign(Effect.SourceBmp);
  PicRect := Rect(0,0,FBitmap.Width, FBitmap.Height);
  If Not FAutoSize Then
  	ResizeFitSize;
  Invalidate;
end;
//****************************************************************************
//   ֱ
//****************************************************************************
procedure TSBImageEffectView.SetVersion(const Value: String);
begin
  FVersion := Value;
end;

//****************************************************************************

{ TSBImage32 }

constructor TSBImage32.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

end;

destructor TSBImage32.Destroy;
begin

  inherited;
end;

procedure TSBImage32.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited;

  If (Button = mbRight) And Not (MouseDragging) then
  Begin
    MouseDragPos := Point(X, Y);
    MouseMovePos := Point(MouseDragPos.x+1, MouseDragPos.Y+1);
    MouseDragging := True;
    Screen.Cursor := crHandPoint;
  End;

end;

procedure TSBImage32.MouseMove(Shift: TShiftState; X, Y: Integer);
Var
	P1, P2: TPoint;
begin
	Inherited;
  if MouseDragging Then
  begin
    // ٷ  簢 
    P1 := MouseDragPos; P2 := MouseMovePos;
    Canvas.DrawFocusRect(Rect(Min(P1.X,P2.X),Min(P1.Y,P2.Y),Max(P1.X,P2.X), Max(P1.Y,P2.Y)));

    // 콺 巡 ġ  ġ  簢 ׸
    P1 := MouseDragPos; P2 := Point(X, Y);
    Canvas.DrawFocusRect(Rect(Min(P1.X,P2.X),Min(P1.Y,P2.Y),Max(P1.X,P2.X), Max(P1.Y,P2.Y)));
    // ֱ 巡 ġ 
    MouseMovePos := Point(X,Y);
  end;
end;

procedure TSBImage32.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
Var
	P1, P2: TPoint;
begin
  inherited;
  if (Button = mbRight) And (MouseDragging) then
  begin
    P1 := MouseDragPos; P2 := Point(X, Y);

    Canvas.DrawFocusRect(Rect(Min(P1.X,P2.X),Min(P1.Y,P2.Y),Max(P1.X,P2.X), Max(P1.Y,P2.Y)));
    MouseDragPos := Point(X, Y);
    MouseDragging := False;
    Screen.Cursor := crDefault;
	End;
end;


{ TSBImageEffectView Class}

//****************************************************************************
// public Execute 'Add color noise'
//****************************************************************************
procedure TSBImageEffectView.Exe_AddColorNoise(Amount: Integer);
begin
	FBitmap.Assign (Effect.Effect_AddColorNoise(Amount));
end;
//****************************************************************************
// public: execute 'Add mono noise'.
//****************************************************************************
procedure TSBImageEffectView.Exe_AddMonoNoise(Amount: Integer);
begin
    FBitmap.Assign (Effect.Effect_AddMonoNoise(Amount));
end;
//****************************************************************************
// public: Exeute 'ani-aliased'
//****************************************************************************
procedure TSBImageEffectView.Exe_AntiAlias;
begin
	FBitmap.Assign (Effect.Effect_AntiAlias);
end;
//****************************************************************************
// public: Blur
//****************************************************************************
procedure TSBImageEffectView.Exe_Blur;
begin
    FBitmap.Assign (Effect.Effect_Blur);
end;
//****************************************************************************
// public: Contrast.
//****************************************************************************
procedure TSBImageEffectView.Exe_Contrast(Amount: Integer);
begin
	FBitmap.Assign (Effect.Effect_Contrast(Amount));
end;
//****************************************************************************
// public Copping.
//****************************************************************************
procedure TSBImageEffectView.Exe_Crop(Bounds: TRect);
begin
	FBitmap.Assign (Effect.Effect_Crop(Bounds));
end;
//****************************************************************************
// public Darkness
//****************************************************************************
procedure TSBImageEffectView.Exe_Darkness(Amount: Integer);
begin
	FBitmap.Assign (Effect.Effect_Darkness(Amount));
end;
//****************************************************************************
// public Emboss.
//****************************************************************************
procedure TSBImageEffectView.Exe_Emboss;
begin
	FBitmap.Assign (Effect.Effect_Emboss);
end;
//****************************************************************************
// public Flip-Horizontally.
//****************************************************************************
procedure TSBImageEffectView.Exe_FlipHorizontal;
begin
	FBitmap.Assign (Effect.Effect_FlipHorizontal);
end;
//****************************************************************************
// public: Flip-Vertically.
//****************************************************************************
procedure TSBImageEffectView.Exe_FlipVertical;
begin
	FBitmap.Assign (Effect.Effect_FlipVertical);
end;
//****************************************************************************
// public: Gaussian blur
//****************************************************************************
procedure TSBImageEffectView.Exe_GaussianBlur(Amount: Integer);
begin
	FBitmap.Assign (Effect.Effect_GaussianBlur(Amount));
end;
//****************************************************************************
// public: Grayscale
//****************************************************************************
procedure TSBImageEffectView.Exe_GrayScale;
begin
	FBitmap.Assign (Effect.Effect_GrayScale);
end;
//****************************************************************************
// public: Invert
//****************************************************************************
procedure TSBImageEffectView.Exe_Invert;
begin
	FBitmap.Assign (Effect.Effect_Invert);
end;
//****************************************************************************
// public: Lightness
//****************************************************************************
procedure TSBImageEffectView.Exe_Brightness(Amount: Integer);
begin
	FBitmap.Assign (Effect.Effect_Lightness(Amount));
end;
//****************************************************************************
// public: Posterize
//****************************************************************************
procedure TSBImageEffectView.Exe_Posterize(Amount: integer);
begin
	FBitmap.Assign (Effect.Effect_Posterize(Amount));
end;
//****************************************************************************
// public: Rotate
//****************************************************************************
procedure TSBImageEffectView.Exe_Rotate(Angle: Integer;
  AntiAlias: Boolean);
begin
	FBitmap.Assign (Effect.Effect_Rotate(Angle, AntiAlias));
end;
//****************************************************************************
// public: Saturation
//****************************************************************************
procedure TSBImageEffectView.Exe_Saturation(Amount: Integer);
begin
	FBitmap.Assign (Effect.Effect_Saturation(Amount));
end;
//****************************************************************************
// public: Sharpen
//****************************************************************************
procedure TSBImageEffectView.Exe_Sharpen;
begin
	FBitmap.Assign (Effect.Effect_Sharpen);
end;
//****************************************************************************
// public: Solarize
//****************************************************************************
procedure TSBImageEffectView.Exe_Solarize(Amount: integer);
begin
	FBitmap.Assign (Effect.Effect_Solarize(Amount));
end;
//****************************************************************************
// public: Split blur
//****************************************************************************
procedure TSBImageEffectView.Exe_SplitBlur(Amount: Integer);
begin
	FBitmap.Assign (Effect.Effect_SplitBlur(Amount));
end;
//****************************************************************************
// public: Tile
//****************************************************************************
procedure TSBImageEffectView.Exe_Tile(Amount: integer);
begin
	FBitmap.Assign (Effect.Effect_Tile(Amount));
end;
//****************************************************************************
// public: Zoom in/out
//****************************************************************************
procedure TSBImageEffectView.Exe_ZoomXY(ScaleX, ScaleY: Single;
  AntiAlias: Boolean);
begin
	FBitmap.Assign (Effect.Effect_ZoomXY(ScaleX, ScaleY, AntiAlias));
end;
//****************************************************************************
// public: UseSource property
//****************************************************************************
function TSBImageEffectView.GetUseOriginalBitmap: boolean;
begin
	if Assigned(Effect) Then
	Result := Effect.UseSource;
end;
//****************************************************************************
// public: UseSource property
//****************************************************************************
procedure TSBImageEffectView.SetUseOriginalBitmap(const Value: boolean);
begin
	if Assigned(Effect) Then
    	Effect.UseSource := Value;
end;
//****************************************************************************
// public: FishEye
//****************************************************************************
procedure TSBImageEffectView.Exe_FishEye(Amount: Integer);
begin
	FBitmap.Assign (Effect.Effect_FishEye(Amount));
end;
//****************************************************************************
// public: Mosaic
//****************************************************************************
procedure TSBImageEffectView.Exe_Mosaic(Amount: Integer);
begin
	FBitmap.Assign(Effect.Effect_Mosaic(Amount));
end;
//****************************************************************************
// public: SplitLight
//****************************************************************************
procedure TSBImageEffectView.Exe_Splitlight(Amount: Integer);
begin
	FBitmap.Assign(Effect.Effect_Splitlight(Amount));
end;
//****************************************************************************
// public; spotlight
//****************************************************************************
procedure TSBImageEffectView.Exe_SpotLight(Amount: integer; Spot: TRect);
begin
	FBitmap.Assign(Effect.Effect_SpotLight(Amount, Spot));
end;
//****************************************************************************
// public: Twist
//****************************************************************************
procedure TSBImageEffectView.Exe_Twist(Amount: Integer);
begin
	FBitmap.Assign(Effect.Effect_Twist(Amount));
end;
//****************************************************************************
// public: Trace.
//****************************************************************************
procedure TSBImageEffectView.Exe_Trace(Amount: integer);
begin
	FBitmap.Assign(Effect.Effect_Trace(Amount));
end;
//****************************************************************************
// public: roate every 90'
//****************************************************************************
procedure TSBImageEffectView.Exe_Rotate90(Angle: integer);
begin
	FBitmap.Assign(Effect.Effect_Rotate90(Angle));
end;
//****************************************************************************
// public: Zoom
//****************************************************************************
procedure TSBImageEffectView.Exe_Zoom(Scale: Single; AntiAlias: Boolean);
begin
	FBitmap.Assign(Effect.Effect_Zoom(Scale, AntiAlias));
end;
//****************************************************************************
// public: return Source Bitmap
//****************************************************************************
function TSBImageEffectView.GetBitmap: TBitmap;
begin
	Result := Effect.SourceBmp;
end;
//****************************************************************************
// Set FCropMode to Value.  FMouse (mCrop) 
//****************************************************************************
procedure TSBImageEffectView.SetCropMode(const Value: Boolean);
begin
  FCropMode := Value;
end;
//****************************************************************************
// public: return CropRect.
//****************************************************************************
Function TSBImageEffectView.GetCropRect: TRect;
begin
	If (CropMode) Then
    	Result := CropRect
    Else
        Result := Rect(0,0,0,0);
End;

end.
